<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\EntityFactories;

use apexl\Io\factories\EntityFactory;
use apexl\Io\modules\user\entities\roleEntity;

/**
 * @method roleEntity create(array $attrs = [], bool $store = true)
 */
readonly class RoleEntityFactory extends EntityFactory
{
    protected function definition(array $attrs): array
    {
        return [
            'name' => $this->faker->name(),
        ];
    }
}