<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\Dtos;

use apexl\dto\Dto;

final readonly class SessionDto extends Dto
{
    public function __construct(
        public int $uid,
        public string $sessionId,
        public bool $active,
        public array $data = [],
    ) {}
}