<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\Request;

use apexl\Io\modules\validation\Interfaces\ValidatesDataInterface;
use apexl\Io\modules\validation\Traits\ValidatesData;
use apexl\Io\Request\IoRequest;

final class MasqueradeRequest extends IoRequest implements ValidatesDataInterface
{
    use ValidatesData;

    public function validationRules(): array
    {
        return [
            'id' => 'required|numeric',
        ];
    }
}